;===============================================================================
;  input/output module: RS232
;===============================================================================
;
;Ports:
;  PD0 = RS232 Rx
;  PD1 = RS232 Tx

.equ	DISPSIZE = 12
;.equ	DISPSIZE = 16

;===============================================================================
; public functions (called from the main program)
;===============================================================================

io_init:
	;initialize I/O interface (called after power-on)
	ldi	temp, 0x02 ;Tx pin = output
	out	DDRD, temp
	ldi	temp, 0x03 ;enable pullup for Rx pin
	out	PORTD, temp
	ldi	temp, 1<<RXEN | 1<<TXEN
	out	UCSRB, temp
	ldi	temp, 53 ;TODO 51 ;9600 Baud
	out	UBRRL, temp
	ret
	
;--------------------

io_update:
	;update display (called after number or operation changed)
	ldi	temp, 0x1b ;clear screen
	rcall	io_putc
	ldi	temp, '['
	rcall	io_putc
	ldi	temp, '2'
	rcall	io_putc
	ldi	temp, 'J'
	rcall	io_putc
	ldi	temp, 0x1b ;cursor to line 1, col 3
	rcall	io_putc
	ldi	temp, '['
	rcall	io_putc
	ldi	temp, '1'
	rcall	io_putc
	ldi	temp, ';'
	rcall	io_putc
	ldi	temp, '3'
	rcall	io_putc
	ldi	temp, 'H'
	rcall	io_putc

	sbrc	flags, Error
	rjmp	io_update_error
	
	;output spaces to right-align number
	push	length
	mov	temp, first
	lsr	temp
	ldi	ZL, num_io
	add	ZL, temp
	mov	temp2, first
	ldi	temp, ' '
	ldi	count, DISPSIZE
	sub	count, length
io_update_space:
	cpi	count, 0
	breq	io_update_num
	cpi	count, 1 ;use last digit before number (if any) for sign
	brne	io_update_space_nosign
	sbrc	flags, num_io_neg
	ldi	temp, '-'
io_update_space_nosign:	
	rcall	io_putc
	ldi	temp, ' '
	rcall	io_putc
	dec	count
	rjmp	io_update_space

io_update_num:
	;output number digits, followed by spaces or the decimal point
	ld	temp, Z
	sbrc	first, 0
	inc	ZL
	sbrs	first, 0
	swap	temp
	andi	temp, 0x0F
	ori	temp, 0x30
	rcall	io_putc
	ldi	temp, ' '
	cpi	temp2, DISPSIZE-1
	brne	PC+2
	ldi	temp, '.'
	rcall	io_putc
	inc	first
	inc	temp2
	dec	length
	brne	io_update_num
	pop	length
	
io_updatesymbols:
	;output next operation and and memory state (zero/nonzero)
	ldi	temp, 0x1b ;move cursor to end of number
	rcall	io_putc
	ldi	temp, '['
	rcall	io_putc
	ldi	temp, '1'
	rcall	io_putc
	ldi	temp, ';'
	rcall	io_putc
	ldi	temp, '0' + (DISPSIZE*2 + 3) / 10
	rcall	io_putc
	ldi	temp, '0' + (DISPSIZE*2 + 3) % 10
	rcall	io_putc
	ldi	temp, 'H'
	rcall	io_putc
	ldi	temp, ' '
	sbrs	flags2, MemZero
	ldi	temp, ''' ;display "MEMORY" dot
	rcall	io_putc
	ldi	temp, ' ' ;output operation
	sbrc	flags, AnyInput
	rjmp	io_update_operation
	sbrc	flags2, OP_EQUAL
	rjmp	io_update_operation
	sbrc	flags2, OP_ADD
	ldi	temp, '+'
	sbrc	flags2, OP_SUB
	ldi	temp, '-'
	sbrc	flags2, OP_MUL
	ldi	temp, '*'
	sbrc	flags2, OP_DIV
	ldi	temp, '/'
io_update_operation:
	rcall	io_putc
	;output sign (if number takes up all numeric digits)
	ldi	temp, 0x1b ;cursor home
	rcall	io_putc
	ldi	temp, '['
	rcall	io_putc
	ldi	temp, 'H'
	rcall	io_putc
	ldi	temp, ' '
	cpi	length, DISPSIZE
	brlo	io_update_sign_end
	sbrc	flags, num_io_neg
	ldi	temp, '-' ;display left-hand minus sign (negative number)
io_update_sign_end:
	rcall	io_putc
	ret
	
io_update_error:
	ldi	temp, 'E'
	rcall	io_putc
	ldi	temp, ' '
	rcall	io_putc
	ldi	temp, 'r'
	rcall	io_putc
	ldi	temp, ' '
	rcall	io_putc
	ldi	temp, 'r'
	rcall	io_putc
	ldi	temp, ' '
	rcall	io_putc
	ldi	temp, 'o'
	rcall	io_putc
	ldi	temp, ' '
	rcall	io_putc
	ldi	temp, 'r'
	rcall	io_putc
	ldi	temp, ' '
	rcall	io_putc
	ldi	count, (DISPSIZE-5)+1
io_update_error_loop:
	rcall	io_putc
	rcall	io_putc
	dec	count
	brne	io_update_error_loop
	ret
	
;--------------------

io_getkey:
	;wait for keypress and return keycode
	rcall	io_getc
	cpi	temp, '0'
	brlo	io_getkey_nonum
	cpi	temp, '9'+1
	brsh	io_getkey_nonum
	andi	temp, 0x0F ;number
	ret
io_getkey_nonum:
	ldi	ZL, LOW(keytable*2)
	ldi	ZH, HIGH(keytable*2)
io_getkey_loop:
	lpm	temp2, Z+
	cpi	temp2, 0
	breq	io_getkey ;no match -> unknown key
	cp	temp, temp2
	lpm	temp2, Z+
	brne	io_getkey_loop
	mov	temp, temp2
	ret
	
;--------------------
	
keytable: ;keyboard mapping (without number keys)
.db '+',  KEY_ADD,     '-',  KEY_SUB,    '*', KEY_MUL,    '/', KEY_DIV
.db '%',  KEY_PERCENT, '=',  KEY_EQUAL,  '_', KEY_NEG,    '.', KEY_DOT
.db 0x1b, KEY_AC,      0x08, KEY_CE,     '\', KEY_INV,    's', KEY_SQRT
.db 'p',  KEY_MEMADD,  'm',  KEY_MEMSUB, 'r', KEY_MEMRCL, 'c', KEY_MEMCLR
.db 0x0a, KEY_EQUAL,   0x0d, KEY_EQUAL, 0, 0

;===============================================================================
;  private functions
;===============================================================================

io_getc:
	;read byte from UART
	sbis	UCSRA, RXC
	rjmp	io_getc
	in	temp, UDR
	ret
	
;--------------------

io_putc:
	;send byte to UART
	sbis	UCSRA, UDRE
	rjmp	io_putc
	out	UDR, temp
	ret
